## Dev Requirements

- [Node.js](https://nodejs.org/en/) (LTS is recommended)
- [yarn](https://yarnpkg.com/getting-started/install)
  - We use `yarn` as package manager instead of `npm`
  - Installation: `sudo npm install --global yarn`
      - `npm` comes with Node.js


## Project Setup

```sh
yarn install
```

`yarn install` installs all dependencies of the package (defined by package.json) in the project directory.
The packages are installed only locally inside the `node_modules` folder.

### Compile and Hot-Reload for Development

```sh
yarn dev
```

To be able to use a remote bff api and a remote keycloak you must set the following, in `frontend/config.json`:
```json
{
  "backendURL": "http://10.200.41.161",
  "ssoURI": "https://ucs-sso-ng.ram.jb41.ucs.icu/"
}
```
Replace the values with your system. The keycloak host must be an fqdn, the ucs apache needs to now the hostname.
You might also need to adjust the `redirect_url` in keycloak for the client.
You can use the `univention-create-keycloak-clients` script to edit that.
````sh
/usr/share/ucs-school-ui-users-frontend/univention-create-keycloak-clients --admin-username admin --admin-pwdfile /etc/keycloak.secret --client-id "school-ui-users-dev" --recreate-client
```


### Type-Check, Compile and Minify for Production

```sh
yarn build
```

### Preview the Production build locally

You need to run `yarn build` beforehand manually
```sh
yarn preview
```


### Run Unit Tests with [Vitest](https://vitest.dev/)

```sh
yarn test:unit
```

### Run End-to-End Tests with [Cypress](https://www.cypress.io/)

```sh
yarn build
yarn test:e2e # or `yarn test:e2e:ci` for headless testing
```

### Type-Check

```sh
yarn type-check
```

### Lint with [ESLint](https://eslint.org/)

```sh
yarn lint
```

To automatically fix some problems you can run

```sh
yarn lint:fix
```

### These commands and potentially more can be found in the `scripts` block in [package.json](package.json)


## IDEs
### VSCode

[VSCode](https://code.visualstudio.com/) + [Volar](https://marketplace.visualstudio.com/items?itemName=Vue.volar) (and disable Vetur) + [TypeScript Vue Plugin (Volar)](https://marketplace.visualstudio.com/items?itemName=Vue.vscode-typescript-vue-plugin).

If the standalone TypeScript plugin doesn't feel fast enough to you, Volar has also implemented a [Take Over Mode](https://github.com/johnsoncodehk/volar/discussions/471#discussioncomment-1361669) that is more performant. You can enable it by the following steps:

1. Disable the built-in TypeScript Extension
   1) Run `Extensions: Show Built-in Extensions` from VSCode's command palette
   2) Find `TypeScript and JavaScript Language Features`, right click and select `Disable (Workspace)`
2. Reload the VSCode window by running `Developer: Reload Window` from the command palette.

### WebStorm

[WebStorm](https://www.jetbrains.com/de-de/webstorm/) + [Vue setup](https://www.jetbrains.com/help/webstorm/vue-js.html#ws_vue_js_before_you_start)
