export interface LabeledValue<T> {
  value: T;
  label: string;
}

// This should be coming from the apiTypes, but is not implemented yet
export type SortingOrder = 'asc' | 'desc' | 'none';
export type Formatter = 'str' | 'unknown';

export interface Column {
  label: string;
  attribute: string;
  // sorting: SortingOrder;
  formatter?: string;
}

export type GlobalAction = 'add';
export interface ListViewConfig {
  url: string;
  allowedGlobalActions: GlobalAction[];
  searchFormOptions: {
    schools: LabeledValue<string>[];
    activationStates: LabeledValue<string>[];
    roles: LabeledValue<string>[];
  };
  columns: Column[];
}

export interface AddViewConfig {
  pages: Page[];
}

export type Access = 'none' | 'read' | 'write';
export type ContextAction = 'delete' | 'edit' | 'set_password' | 'toggle_active' | 'toggle_wifi';

export interface Attribute {
  value?: unknown;
  access: Access;
}
export interface UserListResponseModel {
  id: string;
  allowedActions: ContextAction[];
  attributes: Record<string, Attribute>;
}

export interface FieldInputText {
  type: 'UInputText';
  props: {
    name: string;
    label: string;
    // modelValue: string;
    description?: string;
    required?: boolean;
    access?: 'write' | 'read' | 'none';
  };
}
export interface FieldInputDate {
  type: 'UInputDate';
  props: {
    name: string;
    label: string;
    // modelValue: string;
    description?: string;
    required?: boolean;
    access?: 'write' | 'read' | 'none';
  };
}
export interface FieldInputPassword {
  type: 'UInputPassword';
  props: {
    name: string;
    label: string;
    // modelValue: string;
    description?: string;
    required?: boolean;
    access?: 'write' | 'read' | 'none';
  };
}
export interface FieldInputCheckbox {
  type: 'UInputCheckbox';
  props: {
    name: string;
    label: string;
    // modelValue: string;
    description?: string;
    required?: boolean;
    access?: 'write' | 'read' | 'none';
  };
}
export interface FieldSelect {
  type: 'USelect';
  props: {
    name: string;
    label: string;
    // modelValue: string;
    description?: string;
    required?: boolean;
    access?: 'write' | 'read' | 'none';
    options: {label: string; value: string}[];
  };
}
export interface FieldMultiSelect {
  type: 'UMultiSelect';
  props: {
    name: string;
    label: string;
    // modelValue: string;
    description?: string;
    required?: boolean;
    access?: 'write' | 'read' | 'none';
    options: {label: string; value: string}[];
  };
}
export type Field = FieldInputText | FieldInputDate | FieldInputPassword | FieldInputCheckbox | FieldSelect | FieldMultiSelect;

export interface Fieldset {
  label: string;
  rows: Field[];
}

export interface Page {
  label: string;
  fieldsets: Fieldset[];
}

export type FormValues = Record<string, unknown>;
export type DetailAction = 'save';
export interface UserDetailResponseModel {
  url: string;
  allowedActions: DetailAction[];
  pages: Page[];
  values: FormValues;
}
