import {useLoginStore} from '@/stores/login';


const buildOptions = async function(options: RequestInit, minTokenValidity = 30): Promise<RequestInit> {
  const loginStore = useLoginStore();
  const authHeader = await loginStore.getValidAuthorizationHeader(minTokenValidity);
  if (options.headers !== undefined) {
    options.headers = Object.assign(options.headers, authHeader);
  } else {
    options.headers = authHeader;
  }
  return options;
};


export const fetchAuthenticated = async function(resource: RequestInfo, options = {}): Promise<Response> {
  let updatedOptions = await buildOptions(options);
  let fetchPromise = fetch(resource, updatedOptions);
  const result = await fetchPromise;
  if (result.status === 401) {
    updatedOptions = await buildOptions(options, -1);
    fetchPromise = fetch(resource, updatedOptions);
  }
  return fetchPromise;
};
